summaryrefslogtreecommitdiff
path: root/src/basic
diff options
context:
space:
mode:
authorLuca Boccassi <bluca@debian.org>2023-03-13 21:33:58 +0000
committerLuca Boccassi <bluca@debian.org>2023-03-14 16:37:03 +0000
commit2461943b847d2f17d399b4820c8f37c912871e7d (patch)
tree9033c26e2fe6a5c11673f6a786eb45c218f49788 /src/basic
parent97cff6cd5f58dd7262a0a7e0dec5f515b4fac93f (diff)
downloadsystemd-2461943b847d2f17d399b4820c8f37c912871e7d.tar.gz
log: add iov helpers for log_context
Diffstat (limited to 'src/basic')
-rw-r--r--src/basic/log.c46
-rw-r--r--src/basic/log.h21
2 files changed, 61 insertions, 6 deletions
diff --git a/src/basic/log.c b/src/basic/log.c
index 40602d5dda..8b973f9e0a 100644
--- a/src/basic/log.c
+++ b/src/basic/log.c
@@ -76,6 +76,8 @@ typedef struct LogContext {
/* Depending on which destructor is used (log_context_free() or log_context_detach()) the memory
* referenced by this is freed or not */
char **fields;
+ struct iovec *input_iovec;
+ size_t n_input_iovec;
bool owned;
LIST_FIELDS(struct LogContext, ll);
} LogContext;
@@ -613,7 +615,7 @@ static void log_do_context(struct iovec *iovec, size_t iovec_len, size_t *n) {
assert(iovec);
assert(n);
- LIST_FOREACH(ll, c, _log_context)
+ LIST_FOREACH(ll, c, _log_context) {
STRV_FOREACH(s, c->fields) {
if (*n + 2 >= iovec_len)
return;
@@ -621,6 +623,15 @@ static void log_do_context(struct iovec *iovec, size_t iovec_len, size_t *n) {
iovec[(*n)++] = IOVEC_MAKE_STRING(*s);
iovec[(*n)++] = IOVEC_MAKE_STRING("\n");
}
+
+ for (size_t i = 0; i < c->n_input_iovec; i++) {
+ if (*n + 2 >= iovec_len)
+ return;
+
+ iovec[(*n)++] = c->input_iovec[i];
+ iovec[(*n)++] = IOVEC_MAKE_STRING("\n");
+ }
+ }
}
static int write_to_journal(
@@ -1555,6 +1566,7 @@ LogContext* log_context_attach(LogContext *c) {
assert(c);
_log_context_num_fields += strv_length(c->fields);
+ _log_context_num_fields += c->n_input_iovec;
return LIST_PREPEND(ll, _log_context, c);
}
@@ -1563,8 +1575,9 @@ LogContext* log_context_detach(LogContext *c) {
if (!c)
return NULL;
- assert(_log_context_num_fields >= strv_length(c->fields));
+ assert(_log_context_num_fields >= strv_length(c->fields) + c->n_input_iovec);
_log_context_num_fields -= strv_length(c->fields);
+ _log_context_num_fields -= c->n_input_iovec;
LIST_REMOVE(ll, _log_context, c);
return NULL;
@@ -1583,14 +1596,33 @@ LogContext* log_context_new(char **fields, bool owned) {
return log_context_attach(c);
}
+LogContext* log_context_newv(struct iovec *input_iovec, size_t n_input_iovec, bool owned) {
+ if (!input_iovec || n_input_iovec == 0)
+ return NULL; /* Nothing to do */
+
+ LogContext *c = new(LogContext, 1);
+ if (!c)
+ return NULL;
+
+ *c = (LogContext) {
+ .input_iovec = input_iovec,
+ .n_input_iovec = n_input_iovec,
+ .owned = owned,
+ };
+
+ return log_context_attach(c);
+}
+
LogContext* log_context_free(LogContext *c) {
if (!c)
return NULL;
log_context_detach(c);
- if (c->owned)
+ if (c->owned) {
strv_free(c->fields);
+ iovec_array_free(c->input_iovec, c->n_input_iovec);
+ }
return mfree(c);
}
@@ -1603,6 +1635,14 @@ LogContext* log_context_new_consume(char **fields) {
return c;
}
+LogContext* log_context_new_consumev(struct iovec *input_iovec, size_t n_input_iovec) {
+ LogContext *c = log_context_newv(input_iovec, n_input_iovec, /*owned=*/ true);
+ if (!c)
+ iovec_array_free(input_iovec, n_input_iovec);
+
+ return c;
+}
+
size_t log_context_num_contexts(void) {
size_t n = 0;
diff --git a/src/basic/log.h b/src/basic/log.h
index 0d4956e6b5..c4ac73e27b 100644
--- a/src/basic/log.h
+++ b/src/basic/log.h
@@ -461,10 +461,12 @@ LogContext* log_context_attach(LogContext *c);
LogContext* log_context_detach(LogContext *c);
LogContext* log_context_new(char **fields, bool owned);
+LogContext* log_context_newv(struct iovec *input_iovec, size_t n_input_iovec, bool owned);
LogContext* log_context_free(LogContext *c);
/* Same as log_context_new(), but frees the given fields strv on failure. */
LogContext* log_context_new_consume(char **fields);
+LogContext* log_context_new_consumev(struct iovec *input_iovec, size_t n_input_iovec);
/* Returns the number of attached log context objects. */
size_t log_context_num_contexts(void);
@@ -486,8 +488,15 @@ DEFINE_TRIVIAL_CLEANUP_FUNC(LogContext*, log_context_free);
#define LOG_CONTEXT_PUSH_STRV(strv) \
_LOG_CONTEXT_PUSH_STRV(strv, UNIQ_T(c, UNIQ))
-/* LOG_CONTEXT_CONSUME_STR()/LOG_CONTEXT_CONSUME_STRV() are identical to
- * LOG_CONTEXT_PUSH_STR()/LOG_CONTEXT_PUSH_STRV() except they take ownership of the given str/strv argument.
+#define _LOG_CONTEXT_PUSH_IOV(input_iovec, n_input_iovec, c) \
+ _unused_ _cleanup_(log_context_freep) LogContext *c = log_context_newv(input_iovec, n_input_iovec, /*owned=*/ false);
+
+#define LOG_CONTEXT_PUSH_IOV(input_iovec, n_input_iovec) \
+ _LOG_CONTEXT_PUSH_IOV(input_iovec, n_input_iovec, UNIQ_T(c, UNIQ))
+
+/* LOG_CONTEXT_CONSUME_STR()/LOG_CONTEXT_CONSUME_STRV()/LOG_CONTEXT_CONSUME_IOV() are identical to
+ * LOG_CONTEXT_PUSH_STR()/LOG_CONTEXT_PUSH_STRV()/LOG_CONTEXT_PUSH_IOV() except they take ownership of the
+ * given str/strv argument.
*/
#define _LOG_CONTEXT_CONSUME_STR(s, c, strv) \
@@ -504,3 +513,9 @@ DEFINE_TRIVIAL_CLEANUP_FUNC(LogContext*, log_context_free);
#define LOG_CONTEXT_CONSUME_STRV(strv) \
_LOG_CONTEXT_CONSUME_STRV(strv, UNIQ_T(c, UNIQ))
+
+#define _LOG_CONTEXT_CONSUME_IOV(input_iovec, n_input_iovec, c) \
+ _unused_ _cleanup_(log_context_freep) LogContext *c = log_context_new_consumev(input_iovec, n_input_iovec);
+
+#define LOG_CONTEXT_CONSUME_IOV(input_iovec, n_input_iovec) \
+ _LOG_CONTEXT_CONSUME_IOV(input_iovec, n_input_iovec, UNIQ_T(c, UNIQ))
@@ -461,10 +461,12 @@ LogContext* log_context_attach(LogContext *c);
LogContext* log_context_free(LogContext *c);
-/* Same as log_context_new(), but frees the given fields strv on failure. */
+/* Same as log_context_new(), but frees the given fields strv/iovec on failure. */
LogContext* log_context_new_consume(char **fields);